import React, { useEffect, useMemo, useRef } from "react";
import { register } from "@antv/x6-react-shape";
import { Graph, Node } from "@antv/x6";
import type { ColumnItem, ViewTable } from "@/type";
function TableNode({ node, graph }: { node: Node; graph: Graph }) {
  const { table, tableColumnList } = node.getData<{
    table: ViewTable;
    tableColumnList: ColumnItem[];
  }>();
  const headerRef = useRef<HTMLDivElement>(null);
  const contentRef = useRef<HTMLDivElement>(null);

  // 重置宽高
  useEffect(() => {
    if (table.tableExtInfoDto.width < 0) {
      const width = headerRef.current?.offsetWidth || 0;
      const height = (headerRef.current?.offsetHeight || 0) + (contentRef.current?.offsetHeight || 0);
      if(width && height) {
        node.resize(width, height);
      }
    }
  }, []);

  // 主键字段
  const primaryKeyList = tableColumnList.filter((item) => item.isUnique);

  // 非主键字段
  const otherKeyList = tableColumnList.filter((item) => !item.isUnique);

  const FiledItem = ({ record }: { record: ColumnItem }) => {
    return (
      <div className="w-full flex py-4px px-8px">
        {record.isUnique ? (
          <span className="m-r-4px">
            <i className="iconfont icon-key-fill color-#efc553 m-r--4px" />
            <span className="text-12px color-#f15359">P</span>
            <span className="text-12px color-#8dafd1">F</span>
          </span>
        ) : (
          <span>
            <i className="iconfont icon-24_beizhu color-#efc553 m-r-4px text-12px" />
          </span>
        )}
        <span className="flex-1 truncate">
          {record.schemaName}({record.name}):VARCHAR2(20)
        </span>
      </div>
    );
  };

  return (
    <div className="w-full h-full border border-1px border-solid border-#333 flex flex-col overflow-hidden rounded-8px">
      <div
        ref={headerRef}
        className="
          header 
          border-b-solid 
          border-b-1px 
          border-b-#333 
          truncate 
          bg-#99b4d1 
          text-center 
          py-4px 
          font-bold 
          text-14px 
          min-h-32px 
          px-8px
        "
      >
        {table.schemaName}({table.name})
      </div>
      <div ref={contentRef} className="bg-#fafafa flex-1">
        <div className="primary-key border-b-solid border-b-1px border-b-#333 py-4px min-h-32px">
          {primaryKeyList.map((item) => {
            return <FiledItem record={item} />;
          })}
        </div>
        <div className="field-info">
          {otherKeyList.map((item) => {
            return <FiledItem record={item} />;
          })}
        </div>
      </div>
    </div>
  );
}

register({
  shape: "table-node",
  component: TableNode,
  width: 300,
  height: 220,
  effect: ["data"],
});
